//	DANCETIMESounder.h - The game's sounder.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef DANCETIMESOUNDER_H_
#define DANCETIMESOUNDER_H_

#include "SequencerCursor.h"

#include "synth/PolyphonicSynth.h"
#include "synth/SimpleSynth.h"
#include "synth/CrashSynth.h"
#include "synth/KickSynth2.h"
#include "synth/ClapSynth.h"
#include "synth/HatSynth.h"
#include "sound/Sounder.h"
#include "sound/Delay.h"

///	The game's sounder.
class DANCETIMESounder : public Sounder,
						 public SequencerCursor::Callback
{
  public:
	///	Constructor.
	DANCETIMESounder();
	///	Destructor.
	~DANCETIMESounder();

	///	Called for every audio block.
	void getAudio(float *input, float *output, int numSamples);

	///	Called for each sound file resource by LoadingScreen.
	void loadSoundFileResource(SoundFileLoaderPtr soundFileLoader) {};

	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerHi(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerLo(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerSlow(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerMine(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerPickup(float val);
	///	Triggers a synth note.
	/*!
		\param val A value 0->1 which will be mapped to a particular pitch.
	 */
	void triggerPew(float val);

	///	Triggers a kick.
	void triggerKick(int sample);
	///	Triggers a clap.
	void triggerClap(int sample);
	///	Triggers a hat.
	void triggerHat(int sample);
	///	Triggers a crash.
	void triggerCrash(int sample);

	///	Updates the caterpillar notes.
	void updateNotes(int *val);
	///	Updates the hi notes.
	void updateNotesHi(int *val = 0);
	///	Updates the other notes.
	void updateNotesOther(int *val = 0);

	///	Starts/stops the (non-perc) synths playing.
	void setMusicPlaying(bool val);
	///	Resets the ghosts.
	void resetGhosts();
	///	Turns the synth distortion on/off.
	void setDistortion(bool val);

	///	Sets the individual instruments' amplitudes.
	void setNoteAmp(float player, float ghost1, float ghost2);

	///	Sets the indexed instrument's step and triggers the note immediately.
	/*!
		\param instrument 0: Kick, 1: Clap, 2: Hat, 3: Lo, 4: Hi, 5: Pad
		\param step The step to set.
		\param value Which pitch the step should be (0->8, -1 == no note).
	 */
	void setStep(int instrument, int step, int value);
	///	Clears all step sequences.
	void clear();
	///	Sets the tempo.
	void setTempo(float val);
	///	Mutes the indexed sequence.
	void mute(int instrument, bool val);

	///	Called whenever the cursor moves to the next step in the sequence.
	void nextStepCallback(int step, int sample);

	///	Loads all the available pitches.
	void loadPitches();
	///	Selects a random pitch set (different to the current one).
	void selectRandomPitches();
	///	Selects the indexed pitch set.
	void selectPitches(int index);

	///	Callback class used to tell GamePlayState when a note should be emphasized.
	class Callback
	{
	  public:
		///	Called when a new step occurs.
		virtual void nextStepCallback(int step, int sample) = 0;
	};
	///	Registers a callback to call for each step.
	void registerCallback(Callback *c);
  private:
	///	Some constants.
	enum
	{
		BufferSize = 16384
	};

	///	The hi synth.
	PolyphonicSynth<SimpleSynth> hiSynth;
	///	The hi synth delay.
	Delay hiDelay;

	///	The lo synth.
	PolyphonicSynth<SimpleSynth> loSynth;
	///	The lo synth delay.
	Delay loDelay;
	///	Temporary buffer for the lo synth.
	float loBuffer[BufferSize];

	///	The slow synth
	PolyphonicSynth<SimpleSynth> slowSynth;
	///	The slow synth delay.
	Delay slowDelay;
	///	Temporary buffer for the slow synth.
	float slowBuffer[BufferSize];

	///	The mine synth
	PolyphonicSynth<SimpleSynth> mineSynth;
	///	The slow mine delay.
	Delay mineDelay;
	///	Temporary buffer for the mine synth.
	float mineBuffer[BufferSize];

	///	The pickup synth
	PolyphonicSynth<SimpleSynth> pickupSynth;
	///	The slow mine delay.
	Delay pickupDelay;
	///	Temporary buffer for the mine synth.
	float pickupBuffer[BufferSize];

	///	The pew synth
	PolyphonicSynth<SimpleSynth> pewSynth;
	///	The pew delay.
	Delay pewDelay;
	///	Temporary buffer for the pew synth.
	float pewBuffer[BufferSize];

	///	The Kick synth.
	PolyphonicSynth<KickSynth2> kick;
	///	The Clap synth.
	ClapSynth clap;
	///	The Hat synth.
	HatSynth hat;
	///	The pew delay.
	Delay percDelay;
	///	Temporary buffer for the pew synth.
	float percBuffer[BufferSize];

	///	For triggering notes.
	SequencerCursor cursor;

	///	The individual instruments' amplitudes.
	float amplitudes[3];

	///	Whether to play a kick on each of the 16 steps.
	bool kickTriggers[16];
	///	Whether to play a kick on each of the 16 steps.
	bool clapTriggers[16];
	///	Whether to play a kick on each of the 16 steps.
	bool hatTriggers[16];
	///	When to play a lo synth note.
	int loTriggers[16];
	///	When to play a hi synth note.
	int hiTriggers[16];
	///	When to play a pad synth note.
	int padTriggers[16];

	///	Whether the various sequences are muted or not.
	bool muted[6];

	///	The pad gets triggered at a slower rate than the others.
	int padStep;

	///	Whether or not the music is currently playing.
	bool musicPlaying;
	///	True if the synths are currently being distorted.
	bool distortion;

	///	Used to fade out.
	float fadeOut;

	///	The current pitch set.
	int currentPitches;
	///	The available pitch sets
	std::vector<float> pitches[8];

	///	The callback to call for each step.
	Callback *callback;
};

#endif
